package github.benjamin.equipreservebackend.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import github.benjamin.equipreservebackend.constant.ApprovalStep;
import github.benjamin.equipreservebackend.constant.ReservationStatus;
import github.benjamin.equipreservebackend.dto.LeaderApprovalDTO;
import github.benjamin.equipreservebackend.entity.Approval;
import github.benjamin.equipreservebackend.entity.Reservation;
import github.benjamin.equipreservebackend.entity.Role;
import github.benjamin.equipreservebackend.exception.ApiException;
import github.benjamin.equipreservebackend.mapper.ApprovalMapper;
import github.benjamin.equipreservebackend.mapper.ReservationMapper;
import github.benjamin.equipreservebackend.mapper.RoleMapper;
import github.benjamin.equipreservebackend.service.ApprovalService;
import github.benjamin.equipreservebackend.vo.LeaderApprovalVO;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@RequiredArgsConstructor
public class ApprovalServiceImpl implements ApprovalService {

    private final ApprovalMapper approvalMapper;

    private final ReservationMapper reservationMapper;

    private final RoleMapper roleMapper;

    @Override
    public void addApproval(LeaderApprovalDTO dto) {
        List<Role> userRoles = roleMapper.selectRoleByUserId(dto.getUserId());
        final ApprovalStep step;
        final ReservationStatus status;
        if (userRoles.stream().anyMatch(r -> r.getCode().equals("LEADER"))) {
            step = ApprovalStep.LEADER;
            status = dto.getIsApprove() ? ReservationStatus.APPROVED : ReservationStatus.REJECTED;
        } else if (userRoles.stream().anyMatch(r -> r.getCode().equals("DEVICE_ADMIN"))){
            step = ApprovalStep.DEVICE_ADMIN;
            if (dto.getIsApprove()) {
                status = dto.getNeedAssist() ? ReservationStatus.APPROVED_ASSIST : ReservationStatus.APPROVED;
            } else {
                status = ReservationStatus.REJECTED;
            }
        } else {
            throw new ApiException("用户角色不匹配，请核对或联系开发人员");
        }
        Approval approval = Approval.builder()
                .approverId(dto.getUserId())
                .reservationId(dto.getReservationId())
                .step(step.getStep())
                .decision(dto.getIsApprove() ? 1 : 0)
                .build();
        approvalMapper.insert(approval);
        reservationMapper.update(new LambdaUpdateWrapper<Reservation>()
                .eq(Reservation::getId, dto.getReservationId())
                .set(Reservation::getStatus, status));
    }

    @Override
    public IPage<LeaderApprovalVO> getLeaderApproval(Page<Approval> pageRequest, Long userId, String applicantName, String deviceName) {
        return approvalMapper.getLeaderApproval(pageRequest, userId, applicantName, deviceName);
    }
}
